@@ -3,6 +3,8 @@
 //! 
 //! 
 
+use anyhow::{anyhow};
+
 use num_traits::{Float};
 use num_traits::cast::FromPrimitive;
 
@@ -11,6 +13,7 @@ use indexmap::set::*;
 use std::cmp::Ordering;
 
 use std::sync::Arc;
+use rand::thread_rng;
 
 use quantiles::{ckms::CKMS};     // we could use also greenwald_khanna
 
@@ -18,7 +21,8 @@ use quantiles::{ckms::CKMS};     // we could use also greenwald_khanna
 use hnsw_rs::prelude::*;
 use hnsw_rs::hnsw::{DataId};
 
-use crate::nodeparam::*;
+use crate::tools::{dimension::*,nodeparam::*};
+use rand::distributions::{Distribution};
 
 // morally F should be f32 and f64.  
 // The solution from ndArray is F : Float + AddAssign + SubAssign + MulAssign + DivAssign + RemAssign + Display + Debug + LowerExp + UpperExp + (ScalarOperand + LinalgScalar) + Send + Sync.   
@@ -150,11 +154,78 @@ impl <F> KGraph<F>
         &self.neighbours
     }
 
-    /// get out edges from node
-    pub fn get_out_edges(&self, node : NodeIdx) -> &Vec<OutEdge<F>> {
+    /// get out edges from node given its index
+    pub fn get_out_edges_by_idx(&self, node : NodeIdx) -> &Vec<OutEdge<F>> {
         &self.neighbours[node]
     }
 
+
+    /// given a DataId returns list of edges from corresponding point or None if error occurs
+    pub fn get_out_edges_by_data_id(&self,  data_id: &DataId) -> Result<&Vec<OutEdge<F>>, anyhow::Error> {
+        let idx = self.get_idx_from_dataid(data_id);
+        if idx.is_none() {
+            return Err(anyhow!("bad data_id"));
+        }
+        //
+        let idx = idx.unwrap();
+        return Ok(self.get_out_edges_by_idx(idx));
+    } // end of get_out_edges_by_data_id
+
+
+    /// estimate intrinsic dimension around a point given by its data_id.
+    /// We implement the method described in :  
+    ///     Maximum likelyhood estimation of intrinsic dimension.
+    ///     Levina E. and Bickel P.J NIPS 2004.  [Levina-Bickel](https://www.stat.berkeley.edu/~bickel/mldim.pdf)
+    /// 
+    pub fn intrinsic_dim_at_data_id(&self, data_id : &DataId) -> Result<f64,anyhow::Error>   {
+        //
+        let edges_res = self.get_out_edges_by_data_id(data_id);
+        if edges_res.is_err() {
+            return Err(edges_res.err().unwrap());
+        }
+        let edges: &Vec<OutEdge<F>> = edges_res.unwrap();
+        intrinsic_dimension_from_edges::<F>(edges)
+    } // end of intrinsic_dim
+
+
+    /// We implement the method described in :  
+    ///     Maximum likelyhood estimation of intrinsic dimension.
+    ///     Levina E. and Bickel P.J NIPS 2004.  [Levina-Bickel](https://www.stat.berkeley.edu/~bickel/mldim.pdf).  
+    /// 
+    /// We estimate dimension by sampling sampling_size points around which we estimate intrinsic
+    /// dimension and returs mean and standard deviation if we do not encounter error.
+    ///   
+    /// **Note : As recommended in the Paper cited, the estimation needs more than 20 neighbours around each point.**
+    ///        We provide an estimation even if this condition is not fulfilled but it is less robust.
+    // TODO : get an histogram of dimensions
+    pub fn estimate_intrinsic_dim(&self, sampling_size: usize) ->  Result<(f64,f64),anyhow::Error> {
+        // we sample points, ignoring the probability to sample twice or more the ame point.
+        // TODO sampling without replacement?
+        let mut dims = Vec::<f64>::with_capacity(sampling_size);
+        let nb_nodes = self.get_nb_nodes();
+        let mut rng = thread_rng();
+        let between = rand_distr::Uniform::from(0..nb_nodes);
+        // TODO to be parallelized if necessary
+        for _ in 0..sampling_size {
+            let node = between.sample(&mut rng);
+            let edges = &self.neighbours[node];
+            let dim = intrinsic_dimension_from_edges(edges);
+            if dim.is_ok() {
+                dims.push(dim.unwrap());
+            }
+        }
+        if dims.len() == 0 {
+            log::error!("could not sample dimension");
+            return Err(anyhow!("could not sample points"));
+        }
+        let mean_dim : f64 = dims.iter().sum::<f64>()/dims.len() as f64;
+        let mut sigma = dims.iter().fold(0., |acc, d| acc + (d-mean_dim)*(d-mean_dim));
+        sigma = (sigma/dims.len() as f64).sqrt();
+        log::debug!(" mean dimension : {:.3e}, sigma : {:.3e}, nb_points used: {}", mean_dim, sigma, dims.len());
+        return Ok((mean_dim, sigma));
+    } // end of estimate_intrinsic_dim
+
+
     /// As data can come from hnsw with arbitrary data id not on [0..nb_data] we reindex
     /// them for array computation.  
     /// At the end we must provide a way to get back to original labels of data.
@@ -208,7 +279,9 @@ impl <F> KGraph<F>
             max_in_degree = max_in_degree.max(in_degrees[i]);
             mean_in_degree += in_degrees[i] as f32;
         }
-        mean_in_degree /= in_degrees.len() as f32;
+        if in_degrees.len() > 0 {
+            mean_in_degree /= in_degrees.len() as f32;
+        }
         //
         println!("\n ==========================");
         println!("\n minimal graph statistics \n");
@@ -251,9 +324,8 @@ pub fn kgraph_from_hnsw_all<T, D, F>(hnsw : &Hnsw<T,D>, nbng : usize) -> std::re
     let max_nb_conn = hnsw.get_max_nb_connection() as usize;    // morally this the k of knn bu we have that for each layer
     // check consistency between max_nb_conn and nbng
     if max_nb_conn <= nbng {
-        log::error!("init_from_hnsw_all: number of neighbours must be greater than hnsw max_nb_connection : {} ", max_nb_conn);
-        println!("init_from_hnsw_all: number of neighbours must be greater than hnsw max_nb_connection : {} ", max_nb_conn);
-        return Err(1);
+        log::info!("init_from_hnsw_all: number of neighbours must be less than hnsw max_nb_connection : {} ", max_nb_conn);
+        println!("init_from_hnsw_all: number of neighbours must be less than hnsw max_nb_connection : {} ", max_nb_conn);
     }
     let point_indexation = hnsw.get_point_indexation();
     let nb_point = point_indexation.get_nb_point();
@@ -466,21 +538,31 @@ impl <F> KGraphProjection<F>
     pub fn new<T, D>(hnsw : &Hnsw<T,D>, nbng : usize, layer : usize) -> Self 
                     where T : Clone + Send + Sync,
                           D: Distance<T> + Send + Sync {
-        log::trace!("Projection new ");
+        log::debug!("KGraphProjection new  layer : {}", layer);
         let mut nb_point_to_collect = 0;
         let mut nb_point_below_nbng = 0;
         let max_nb_conn = hnsw.get_max_nb_connection() as usize;
         let max_level_observed = hnsw.get_max_level_observed() as usize;
-        log::trace!("max level observed : {}", max_level_observed);
+        log::debug!("max level observed : {}", max_level_observed);
         // check number of points kept in
+        if layer >= max_level_observed {
+            log::error!("KGraphProjection::new, layer argument greater than nb_layer!!, layer : {}", layer);
+            println!("KGraphProjection::new, layer argument greater than nb_layer!!, layer : {}", layer);
+
+        }
         for l in (layer..=max_level_observed).rev() {
             nb_point_to_collect += hnsw.get_point_indexation().get_layer_nb_point(l);
+            log::trace!(" layer : {}, nb points to collect : {}", l, nb_point_to_collect);
+        }
+        if nb_point_to_collect <= 0 {
+            log::error!("!!!!!!!!!!!! KGraphProjection cannot collect points !!!!!!!!!!!!!, check layer argument");
+            println!("!!!!!!!!!!!! KGraphProjection cannot collect points !!!!!!!!!!!!!, check layer argument");
         }
         // let _points = Vec::<std::sync::Arc<Point<F>>>::with_capacity(nb_point_to_collect);
         // let _distances = Array2::<F>::zeros((nb_point_to_collect,nb_point_to_collect));
         //
         let layer_u8 = layer as u8;
-        log::info!("Projection : number of point to collect : {}", nb_point_to_collect);
+        log::debug!("Projection : number of point to collect : {}", nb_point_to_collect);
         let mut upper_graph_neighbours = Vec::<Vec<OutEdge<F>>>::with_capacity(nb_point_to_collect);
         for _i in 0..nb_point_to_collect {
             upper_graph_neighbours.push(Vec::<OutEdge<F>>::new());
@@ -767,7 +849,6 @@ fn fix_points_with_no_projection<T,F>(proj_data : &mut HashMap<NodeIdx, OutEdge<
 
 
 
-// ================================================================================
 
 
 mod tests {
@@ -784,8 +865,11 @@ use rand::prelude::*;
 
 #[allow(dead_code)]
 fn log_init_test() {
-    let _ = env_logger::builder().is_test(true).try_init();
-}  
+    let res = env_logger::builder().is_test(true).try_init();
+    if res.is_err() {
+        println!("could not init log");
+    }
+}  // end of log_init_test
 
 
 #[allow(unused)]
@@ -802,7 +886,8 @@ fn gen_rand_data_f32(nb_elem: usize , dim:usize) -> Vec<Vec<f32>> {
 }
 
 
-/// test conversion of full hnsw to KGraph
+/// test conversion of full hnsw to KGraph and dimension estimation. 
+/// mean intrinsic dimension should be around 30 as it is the dimension we use generate random unifrom data 
 #[test]
 fn test_full_hnsw() {
     //
@@ -810,8 +895,9 @@ fn test_full_hnsw() {
     //
     let nb_elem = 20000;
     let dim = 30;
-    let knbn = 10;
+    let knbn = 20;
     //
+    log::debug!("test_full_hnsw");
     println!("\n\n test_serial nb_elem {:?}", nb_elem);
     //
     let data = gen_rand_data_f32(nb_elem, dim);
@@ -830,6 +916,17 @@ fn test_full_hnsw() {
     let kgraph : KGraph<f32> = kgraph_from_hnsw_all(&hns, knbn).unwrap();
     log::info!("minimum number of neighbours {}", kgraph.get_max_nbng());
     let _kgraph_stats = kgraph.get_kraph_stats();
+    // make a test for dimension estimation
+    let id = 10;
+    let dimension = kgraph.intrinsic_dim_at_data_id(&id).unwrap();
+    println!("dimension around point : {}, dim = {:.3e}", id, dimension);
+    log::info!("\n dimension around point : {}, dim = {:.3e}", id, dimension);
+    //
+    let dimension =  kgraph.estimate_intrinsic_dim(10000);
+    assert!(dimension.is_ok());
+    let dimension = dimension.unwrap();
+    log::info!("\n estimation of dimension : {:.3e}, sigma : {:.3e} ", dimension.0 , dimension.1);
+    println!("\n estimation of dimension : {:.3e}, sigma : {:.3e} ", dimension.0 , dimension.1);
 }  // end of test_full_hnsw
 
 
@@ -909,7 +1006,7 @@ fn test_graph_projection() {
     hns.parallel_insert(&data_with_id);
     hns.dump_layer_info();
     //
-    let _graph_projection = KGraphProjection::<f32>::new(&hns, layer , knbn);
+    let _graph_projection = KGraphProjection::<f32>::new(&hns, knbn , layer);
 
 
 } // end of test_graph_projection
