@@ -15,11 +15,9 @@ pub mod hdbscan;
 pub mod embedder;
 pub mod embedparams;
 pub mod graphlaplace;
-pub mod nodeparam;
 pub mod diffmaps;
 pub mod prelude;
 
-pub mod io;
 
 
 lazy_static! {
