@@ -3,4 +3,4 @@
 pub use crate::embedder::*;
 pub use crate::fromhnsw::*;
 pub use crate::embedparams::*;
-pub use crate::io::*;
\ No newline at end of file
+pub use crate::tools::io::*;
\ No newline at end of file
