@@ -2,4 +2,7 @@
 
 pub mod svdapprox;
 pub mod entropy;
-pub mod dichotomy;
\ No newline at end of file
+pub mod dichotomy;
+pub mod io;
+pub mod dimension;
+pub mod nodeparam;
