@@ -31,13 +31,12 @@
 
 
 
-
 use rand_distr::{Distribution, StandardNormal};
 use rand_xoshiro::Xoshiro256PlusPlus;
 use rand_xoshiro::rand_core::SeedableRng;
 
 
-use ndarray::{Dim, Array, Array1, Array2, ArrayBase, Dimension, ArrayView, ArrayViewMut1, ArrayView2 , Ix1, Ix2};
+use ndarray::{Dim, Array, Array1, Array2, ArrayBase, Dimension, ArrayView, ArrayView1, ArrayViewMut1, ArrayView2 , Ix1, Ix2};
 
 // pub to avoid to re-import everywhere explicitly
 pub use ndarray_linalg::{Scalar, Lapack};
@@ -49,6 +48,9 @@ use std::marker::PhantomData;
 use num_traits::float::*;    // tp get FRAC_1_PI from FloatConst
 use num_traits::cast::FromPrimitive;
 
+use rayon::iter::{IntoParallelIterator, ParallelIterator};
+use parking_lot::RwLock;
+
 use sprs::{prod, CsMat, TriMat, CsMatView};
 
 struct RandomGaussianMatrix<F:Float> {
@@ -130,7 +132,7 @@ pub struct MatRepr<F> {
 
 
 impl <F> MatRepr<F> where
-    F: Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + Default  {
+    F: Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + Default + std::marker::Sync {
 
     /// initialize a MatRepr from an Array2
     #[inline]
@@ -195,7 +197,7 @@ impl <F> MatRepr<F> where
 
     
     /// Matrix Vector multiplication. We use raw interface to get Blas.
-    pub fn mat_dot_vector(&self, vec : &Array1<F>) -> Array1<F>  {
+    pub fn mat_dot_vector(&self, vec : &ArrayView1<F>) -> Array1<F>  {
         match &self.data {
             MatMode::FULL(mat) => { return mat.dot(vec);},
             MatMode::CSR(csmat) =>  {
@@ -252,17 +254,16 @@ pub fn transpose_dense_mult_csr<F>(qmat : &Array2<F>, csrmat : &CsMat<F>) -> Arr
     let (qm_r, qm_c) = qmat.dim();             // we expect qm_c to be <= 10 as it corresponds to a rank approximated matrix
     let (csc_r , csc_c) = cscmat.shape();
     assert_eq!(csc_c, qm_r);
-            //  let mut bt = Array2::<F>::zeros((csc_r, qm_c));
-    let mut b =  Array2::<F>::zeros((qm_c, csc_r));
+    let mut bt = Array2::<F>::zeros((csc_r, qm_c));
+//    let mut b =  Array2::<F>::zeros((qm_c, csc_r));
     // we transpose to get the right dimension in csc_mulacc_dense_colmaj (see the documentation for t() in ndarray)
-    b.swap_axes(0,1);
-    prod::csc_mulacc_dense_colmaj(cscmat, qmat.view(), b.view_mut());
+//    b.swap_axes(0,1);
+    prod::csc_mulacc_dense_colmaj(cscmat, qmat.view(), bt.view_mut());
     log::trace!("transpose_dense_mult_csr returning  ({},{}) matrix", csc_r, qm_c);
             // We want a Owned matrix in the STANDARD LAYOUT!! 
             // Array::from_shape_vec(bt.t().raw_dim(), bt.t().iter().cloned().collect()).unwrap()
     // we retranspose ! 
-    b.swap_axes(0,1);
-    b
+    bt.reversed_axes().as_standard_layout().to_owned()
 } // end of small_dense_mult_csr
 
 
@@ -343,7 +344,7 @@ pub struct RangeApprox<'a, F: Scalar> {
 
 /// Lapack is necessary here beccause of QR_ traits coming from Lapack
 impl <'a, F > RangeApprox<'a, F> 
-     where  F : Float + Scalar + Lapack + ndarray::ScalarOperand  + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + num_traits::MulAdd + Default {
+     where  F : Send + Sync + Float + Scalar + Lapack + ndarray::ScalarOperand  + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + num_traits::MulAdd + Default {
 
     /// describes the problem, matrix format and range approximation mode asked for.
     pub fn new(mat : &'a MatRepr<F>, mode : RangeApproxMode) -> Self {
@@ -371,7 +372,7 @@ impl <'a, F > RangeApprox<'a, F>
             },
         };
         //
-        if log::log_enabled!(log::Level::Debug) {
+        if log::log_enabled!(log::Level::Trace) {
             log::debug!("\n checking approximation");
             let delta = check_range_approx_repr(self.mat, &approximator);
             let initial_l2 = norm_frobenius_repr(self.mat);
@@ -385,6 +386,7 @@ impl <'a, F > RangeApprox<'a, F>
 
 
 
+/// 
 /// Given a (m,n) matrice A, this algorithm returns a (m,l) orthogonal matrix Q approximation the range of input. 
 /// l is the asked rank and nb_iter is a number of iterations.
 /// 
@@ -396,7 +398,7 @@ impl <'a, F > RangeApprox<'a, F>
 // TODO Oversampling between 5 and 10 ?
 // Nota : if nbiter == 0 We get Tropp Algo 4.1 or Algo 2.1 of Wei-Zhang-Chen
 pub fn subspace_iteration_full<F> (mat : &Array2<F>, rank : usize, nbiter : usize) -> Array2<F>
-            where F : Float + Scalar  + Lapack + ndarray::ScalarOperand {
+            where F : Send + Sync + Float + Scalar  + Lapack + ndarray::ScalarOperand {
     //
     let mut rng = RandomGaussianGenerator::<F>::new();
     let data_shape = mat.shape();
@@ -413,7 +415,8 @@ pub fn subspace_iteration_full<F> (mat : &Array2<F>, rank : usize, nbiter : usiz
     let layout = MatrixLayout::C { row: m as i32, lda: l as i32 };
     // do first QR decomposition of y and overwrite it
     do_qr(layout, &mut y_m_l);
-    for _j in 1..nbiter {
+    for j in 1..nbiter {
+        log::debug!("svdapprox::subspace_iteration_full iter : {}", j);
         // data.t() * y
         ndarray::linalg::general_mat_mul(F::one() , &mat.t(), &y_m_l, F::zero(), &mut y_n_l);
         // qr returns a (n,n)
@@ -428,7 +431,7 @@ pub fn subspace_iteration_full<F> (mat : &Array2<F>, rank : usize, nbiter : usiz
 }  // end of subspace_iteration_full
 
 
-
+///
 /// Given a (m,n) matrice A, this algorithm returns a (m,l) orthogonal matrix Q approximation the range of input. 
 /// l is the asked rank and nb_iter is a number of iterations.
 /// 
@@ -437,7 +440,7 @@ pub fn subspace_iteration_full<F> (mat : &Array2<F>, rank : usize, nbiter : usiz
 /// It implements the QR iterations as descibed in Algorithm 4.4 from Halko-Tropp
 /// 
 pub fn subspace_iteration_csr<F> (csrmat: &CsMat<F>, rank : usize, nbiter : usize) -> Array2<F>
-            where F : Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc {
+            where F : Send + Sync + Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc {
     //
     log::debug!("in svdapprox::subspace_iteration_csr rank: {:?}, nbiter : {:?}", rank, nbiter);
     //
@@ -459,7 +462,8 @@ pub fn subspace_iteration_csr<F> (csrmat: &CsMat<F>, rank : usize, nbiter : usiz
     let layout = MatrixLayout::C { row: m as i32, lda: l as i32 };
     // do first QR decomposition of y and overwrite it
     do_qr(layout, &mut y_m_l);
-    for _j in 1..nbiter {
+    for j in 1..nbiter {
+        log::debug!("svdapprox::subspace_iteration_csr iter : {}", j);
         // data.t() * y
         y_n_l.fill(F::zero());
         prod::csc_mulacc_dense_rowmaj(csrmat.transpose_view(), y_m_l.view(), y_n_l.view_mut());
@@ -492,12 +496,13 @@ pub fn subspace_iteration_csr<F> (csrmat: &CsMat<F>, rank : usize, nbiter : usiz
     //
 
 #[cfg_attr(doc, katexit::katexit)]
+///
 /// If mat is a (m,n) matrix this function returns an orthonormal matrix Q of dimensions (m,l) such that :
 /// $$ || mat - Q*Q^{t}*mat || < ε$$  with probability at least $$ 1. - min(m,n) 10^{-r} $$
 ///  
-///  - epsil is the l2 norm of the last block of r columns vectors added in Q.
+///  - ε is the l2 norm of the last block of r columns vectors added in Q.
 ///  - r is the number of random vectors sampled to initialize the orthogonalization process.
-///    A rule of thumb is to use r between 5 ad 10. The higher the more cpu is required.
+///    A rule of thumb is to use r between 5 and 10. The higher the more cpu is required.
 ///  - max_rank is the maximum rank asked for.
 /// 
 /// 
@@ -513,7 +518,7 @@ pub fn subspace_iteration_csr<F> (csrmat: &CsMat<F>, rank : usize, nbiter : usiz
 /// Algorithm : Adaptive Randomized Range Finder algo 4.2. from Halko-Martinsson-Tropp 2011
 /// 
 pub fn adaptative_range_finder_matrep<F>(mat : &MatRepr<F> , epsil:f64, r : usize, max_rank : usize) -> Array2<F> 
-        where F : Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc + 
+        where F : Float + Scalar  + Lapack + ndarray::ScalarOperand + sprs::MulAcc + Sync + Send +
                     num_traits::MulAdd +for<'r> std::ops::MulAssign<&'r F> + Default {
     //
     log::debug!("\n  in adaptative_range_finder_matrep, mat shape {:?}, epsil {:.3e}, r : {} , max_rank {}", mat.shape(), epsil, r, max_rank);
@@ -526,6 +531,10 @@ pub fn adaptative_range_finder_matrep<F>(mat : &MatRepr<F> , epsil:f64, r : usiz
     let mut q_mat = Vec::<Array1<F>>::new();         // q_mat stores vectors of size m
     // adjust stop_val so that stopping ccriteria provide a relative approximation
     let stop_val  = epsil/(10. * (2. / f64::FRAC_1_PI()).sqrt());
+    log::debug!(" adaptative_range_finder_matrep stop_val : {}", stop_val);
+    let proba_failure = 1.0E-3;
+    let block_iter = ((m as f64/ proba_failure).ln()/ 10.0f64.ln()) as usize;
+    log::info!(" adaptative_range_finder_matrep suggestion for block_iter {} ", block_iter);
     // 
     // we store omaga_i vector as row vector as Rust has C order it is easier to extract rows !!
     let mut omega = rng.generate_matrix(Dim([data_shape[1], r]));    //  omega is (n, r)
@@ -534,14 +543,13 @@ pub fn adaptative_range_finder_matrep<F>(mat : &MatRepr<F> , epsil:f64, r : usiz
     omega.mat *= coeff_norm;
     // We could store Y = data * omega as matrix (m,r), but as we use Y column,
     // we store Y (as Q) as a Vec of Array1<f64>
-    let mut y_vec =  Vec::<Array1<F>>::with_capacity(r);
-    for j in 0..r {
+    let y_vec : Vec<RwLock<Array1<F>>> = (0..r).map(|j| {
+        // we need to_owned to get a slice later
         let c = omega.mat.column(j).to_owned();
-        let y_tmp = mat.mat_dot_vector(&c);
-        y_vec.push(y_tmp);
-    }
+        RwLock::new(mat.mat_dot_vector(&c.view())) }).collect();
+
     // This vectors stores L2-norm of each Y  vector of which there are r
-    let mut norms_y : Array1<F> = (0..r).into_iter().map( |i| norm_frobenius_full(&y_vec[i].view())).collect();
+    let mut norms_y : Array1<F> = (0..r).into_iter().map( |i| norm_frobenius_full(&y_vec[i].read().view())).collect();
     assert_eq!(norms_y.len() , r); 
     log::debug!(" norms_y : {:.3e}",norms_y);
     //
@@ -561,36 +569,38 @@ pub fn adaptative_range_finder_matrep<F>(mat : &MatRepr<F> , epsil:f64, r : usiz
     while norm_sup_y > &stop_val && nb_iter <= max_iter && q_mat.len() < max_rank {
         // numerical stabilization
         if q_mat.len() > 0 {
-            orthogonalize_with_q(&q_mat[0..q_mat.len()], &mut y_vec[j].view_mut());
+            orthogonalize_with_q(&q_mat[0..q_mat.len()], &mut y_vec[j].write().view_mut());
         }
         // get norm of current y vector
-        let n_j =  norm_frobenius_full(&y_vec[j].view());
+        let n_j =  norm_frobenius_full(&y_vec[j].read().view());
         if n_j < ndarray_linalg::Scalar::sqrt(F::epsilon()) {
             log::info!("adaptative_range_finder_matrep returning  at nb_iter {} with n_j {:.3e} and rank {:?} ", nb_iter, n_j, q_mat.len());
             break;
         }
-        let q_j = &y_vec[j] / n_j;
+        let q_j = &y_vec[j].write().view_mut()/ n_j;
         // we add q_j to q_mat so we consumed on y vector
         q_mat.push(q_j.clone());
         // we make another y, first we sample a new omega_j vector of size n
         let mut omega_j_p1 = rng.generate_stdn_vect(Ix1(data_shape[1]));
         omega_j_p1 *= coeff_norm;
-        let mut y_j_p1 = mat.mat_dot_vector(&omega_j_p1);
+        let mut y_j_p1 = mat.mat_dot_vector(&omega_j_p1.view());
         // we orthogonalize new y with all q's i.e q_mat
         orthogonalize_with_q(&q_mat, &mut y_j_p1.view_mut());
         // the new y will takes the place of old y at rank j%r so we always have the last r y that have been sampled
-        assert_eq!(y_vec[j].len(), y_j_p1.len());
-        y_vec[j].assign(&y_j_p1);
-        // we orthogonalize old y's with new q_j.  CAVEAT Can be made // if necessary 
-        for k in 0..r {
+        assert_eq!(y_vec[j].read().len(), y_j_p1.len());
+        y_vec[j].write().assign(&y_j_p1);
+        // we orthogonalize old y's with new q_j.
+        (0..r).into_par_iter().for_each(|k| {
             if k != j {
                 // avoid k = j as the j vector is the new one
-                let prodq_y = &q_j * q_j.view().dot(&y_vec[k]);
-                y_vec[k] -= &prodq_y;
+                let prodq_y = &q_j * q_j.view().dot(&y_vec[k].read().view());
+                *y_vec[k].write() -= &prodq_y;    
             }
-        }
+        });
         // we update norm_sup_y
-        norms_y[j] = norm_frobenius_full(&y_vec[j].view());
+        for i in 0..r {
+            norms_y[i] = norm_frobenius_full(&y_vec[i].read().view());
+        }
         norm_sup_y = norms_y.iter().max_by(|x,y| x.partial_cmp(y).unwrap()).unwrap();
         if log::log_enabled!(log::Level::Debug) {
             if nb_iter % (max_rank /10).max(1) == 0 { 
@@ -608,11 +618,12 @@ pub fn adaptative_range_finder_matrep<F>(mat : &MatRepr<F> , epsil:f64, r : usiz
     log::debug!("range finder returning a a matrix ({}, {})", m, q_mat.len());
     //  method uninit from version 0.15.0 and later
     let mut q_as_array2  = Array2::uninit((m, q_mat.len()));     // as sprs wants ndarray 0.14.0
-    for i in 0..q_mat.len() {
-        for j in 0..m {
+    for j in 0..m {
+        for i in 0..q_mat.len() {
             q_as_array2[[j,i]] = std::mem::MaybeUninit::new(q_mat[i][j]);
         }
     }
+    log::debug!("\n exiting adaptative_range_finder_matrep"); 
     // we return an array2 where each row is a data of reduced dimension
     unsafe{ q_as_array2.assume_init()}
 } // end of adaptative_range_finder_csmat
@@ -711,7 +722,7 @@ pub struct SvdApprox<'a, F: Scalar> {
 
 
 impl <'a, F> SvdApprox<'a, F>  
-     where  F : Float + Lapack + Scalar  + ndarray::ScalarOperand + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + num_traits::MulAdd + Default {
+     where  F : Send + Sync + Float + Lapack + Scalar  + ndarray::ScalarOperand + sprs::MulAcc + for<'r> std::ops::MulAssign<&'r F> + num_traits::MulAdd + Default {
     
     pub fn new(data : &'a MatRepr<F>) -> Self {
         SvdApprox{data}
@@ -720,6 +731,7 @@ impl <'a, F> SvdApprox<'a, F>
     /// direct svd from Algo 5.1 of Halko-Tropp
     /// Returns an error if either the preliminary range_approximation or the partial svd failed, else returns a SvdResult
     pub fn direct_svd(&mut self, parameters : RangeApproxMode) -> Result<SvdResult<F>, String> {
+        log::debug!("in SvdApprox::direct_svd");
         let ra = RangeApprox::new(self.data, parameters);
         let q;
         let q_opt = ra.get_approximator();
@@ -733,9 +745,9 @@ impl <'a, F> SvdApprox<'a, F>
         let mut b = match &self.data.data {
             MatMode::FULL(mat) => { q.t().dot(mat)},
             MatMode::CSR(mat)  => { 
-                                    log::trace!("direct_svd got csr matrix");
-                                    transpose_dense_mult_csr(&q, mat)
-                                },
+                log::trace!("direct_svd got csr matrix");
+                transpose_dense_mult_csr(&q, mat)
+            },
         };
         //
         let layout = MatrixLayout::C { row: b.shape()[0] as i32, lda: b.shape()[1] as i32 };
@@ -778,6 +790,8 @@ impl <'a, F> SvdApprox<'a, F>
             s_vt = None;
         }
         //
+        log::debug!("end of SvdApprox::do_svd");
+        //
         Ok(SvdResult{s : Some(s), u : s_u, vt : s_vt})
     } // end of do_svd
 
@@ -813,13 +827,13 @@ pub fn norm_frobenius_repr<F>(mat : &MatRepr<F>) -> F
         //
     let norm_l2 = match &mat.data {
         MatMode::FULL(mat)      =>  {
-                                        let norm_l2 = norm_frobenius_full(&mat.view());
-                                        norm_l2
-                                    },
+            let norm_l2 = norm_frobenius_full(&mat.view());
+            norm_l2
+        },
         MatMode::CSR(csr_mat)   => {
-                                        let norm_l2 = norm_frobenius_csmat(&csr_mat.view());
-                                        norm_l2
-                                    },
+            let norm_l2 = norm_frobenius_csmat(&csr_mat.view());
+            norm_l2
+        },
     };
     norm_l2
 } // end of norm_frobenius_repr
@@ -1515,7 +1529,7 @@ fn check_transpose_dense_mult_csr() {
 
 
 #[test]
-fn check_transpose_owned() {
+fn check_transpose_owned_and_view() {
     //
     log_init_test();
     //
@@ -1528,11 +1542,28 @@ fn check_transpose_owned() {
     let check = mat.get_csr().unwrap().get(0,4).unwrap();
     log::debug!("old value should be 2  : {}", check);
     assert!((check- 2.).abs() < 1.0E-10); 
-     // check transposed is correct
-     let check = transposed_csr.get(4,0).unwrap(); 
-     log::debug!("transposed value should be 2  : {}", check);
-     assert!((check - 2.).abs() < 1.0E-10); 
-      
+    // check transposed is correct
+    let check = transposed_csr.get(4,0).unwrap(); 
+    log::debug!("transposed value should be 2  : {}", check);
+    assert!((check - 2.).abs() < 1.0E-10); 
+    // now chech transpose_view
+    let csr = get_wiki_csr_mat_f32();
+    let transposed_csr = csr.transpose_view();
+    // check we get indexes transposed
+    let check = transposed_csr.get(4,0).unwrap(); 
+    log::debug!("transposed in transposed view value should be 2  : {}", check);
+    assert!((check - 2.).abs() < 1.0E-10); 
+    let row = 0;
+    let col_range = transposed_csr.indptr().outer_inds_sz(row);
+    log::debug!("transposed view row i : {}, col_range : {:?}", row, col_range);            
+    let col_range = transposed_csr.indptr().outer_inds_sz(row);
+    // check indptr did not change
+    assert_eq!(col_range, 0..2);
+    for k in col_range {
+        let j = transposed_csr.indices()[k];
+        let w = transposed_csr.data()[k];
+        log::debug!("transposed view of row  : {}, k  : {}, col {}, w {}", row, k, j ,w);
+    }
 } // end of check_transpose_owned
 
 
